%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Kline and Santos (2012)                  %
%     Code for Inference on Pseudo-True Parameter    %
%      Used for Figures 4, 5 of paper                %
%  Fastest when run with parallel toolbox activated  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First: Declare parameters, get data %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

clear;

% Declare the parameters we will be using
R = 1000;          % Number of bootstrap replications.
alpha = 0.05;   % 1 - coverage for the confidence interval
Buff = 0.001;  % How much are we substracting from the sample kbar

% Reset the seed for the random number generator
rand('seed',0); rand('state',0);

% Declare quantile Range - and also k range + partition of grid
tau_l = .1; tau_u = .9;
Dtau = 0.05; DK = 0.01;
K_l = 0; K_u = NaN(int8(1+(tau_u-tau_l)/Dtau),1);

% Set linprog options
options = optimset('display','off'); options.MaxIter = 2000;

yr=90;
K=.05;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Second: Organize data, bins and bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

disp(['Loading and reshaping data for ', num2str(yr)])    

% Get data and back out the bins
% Note D is dummy for missing in code in paper D is dummy for nonmissing

eval(['[Y,X,D,n,dk] = getdata(' int2str(yr) ');']);

% Make Y >0 to enable sparse matrix calculations,  won't affect slope
Y = Y+5; 

% Figure out what are the demographic cells/bins
bins = unique(X, 'rows'); totbins = size(bins,1);
bincount = NaN(totbins,1); binind = zeros(n,1);
totobs = length(D); Mprob = NaN(totbins,1); 

% Figure out how many observations per bin, and who belongs where
parfor i = 1:totbins
    % How many observations per bin
    bincount(i) = sum(ismember(X,bins(i,:),'rows'));    
    binind = binind + (ismember(X,bins(i,:),'rows') == 1)*i;     
end

% And the missing probs per bin
parfor b = 1:totbins
    relD = D(binind == b);
    Mprob(b) = mean(relD);
end

% Create weights
weight = bincount/sum(bincount);
    
% Create a matrix relY that stacks bins in rows, obs in columns only
% contains observations of Ys that were not missing. Also bootstrap Ws
tot = max(bincount);
totnonmiss = int32(max((1-Mprob).*bincount));
relY = NaN(totbins,totnonmiss); 
S = NaN(totbins,tot);

% Preallocate bootstrap weights and put in array form
TempY = NaN; TempD = NaN;

% Reshape Y matrix into a row of nonmissing for each bin - pad with zeros
parfor b = 1:totbins
    % Back out the Ys that are not missing for this bin
    TempY = Y(binind == b); TempD = D(binind == b); TempY = sort(TempY(TempD == 0));

    % Store them
    S(b,:) = [ones(1,bincount(b)) zeros(1,tot - bincount(b))];
    relY(b,:) = [TempY' zeros(1,totnonmiss- length(TempY))];
end
relY = sparse(relY);
S = sparse(S);          % index of observations
S0 = spones(relY);      % index of nonmissing observations

% We'll use 1990 to figure out Ku(tau)
if yr == 90
    for i = 1:length(K_u)
        % Figure out who the relevant tau is and sample bounds
        reltau = tau_l + Dtau*(i-1);
        %sampbound = min([1-reltau reltau min((1-Mprob).*(1-reltau)./Mprob) min(reltau*(1-Mprob)./Mprob)]);
        %sampbound = min([1-reltau reltau min((1-reltau)./Mprob) min(reltau./Mprob)]);
        sampbound = min([min((1-reltau)./(ones(totbins,1)-Mprob)) min(reltau./(ones(totbins,1)-Mprob)) min((1-reltau)./Mprob) min(reltau./Mprob)]);
        K_u(i) = sampbound - Buff;
    end       
end

% And finally, we'll put an absolute upper bound of 0.3
K_u(K_u>= 0.3) = 0.3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Third: Calculate the estimated bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Getting Sample Bounds for each quantile')
%for tau=tau_l:Dtau:tau_u
tau=.5;
% Translate into a tau position/index.
itau = int8(1+(tau-tau_l)/Dtau); 
        
% Figure out dimensions of what we are doing
if (itau==1)
    % Find out the total number of tau iterations
    maxitau = int8(1+(tau_u-tau_l)/Dtau);    
    maxiK = int16(1+(max(K_u)-K_l)/DK);    
    
    % Declare variables to save pseudo trues
    pmin = NaN(size(X,2),maxitau,maxiK);
    pmax = NaN(size(X,2),maxitau,maxiK);
end



% Translate into a k position/index
iK = int8(1+(K-K_l)/DK);

% Calculate the bounds at each cell
cLhat = f_binbounds(relY,tau - (tau+K*(ones(totbins,1)-Mprob)).*Mprob,bincount);
cUhat = f_binbounds(relY,tau - (tau-K*(ones(totbins,1)-Mprob)).*Mprob,bincount);

% If we asked for cs outside constraints, adjust
if any(isnan(cLhat))|any(isnan(cUhat))
    error('Sample bounds must be finite: lower k or restrict to narrower range of tau')
end

% Run the linprog on this thing - Set other parameters
f = [1 zeros(1, size(X,2)-1)]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
A = []; b = []; Aeq = []; beq = [];
LQ = cLhat; UQ = cUhat;

% And find the beta lower and upper bounds
[emin] = linprog( f, A, b, Aeq, beq, LQ, UQ,[],options);
[emax] = linprog(-f, A, b, Aeq, beq, LQ, UQ,[],options);

% Back out the pseudo true parameters implied by this and adjust approx
pmin = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emin;
pmax = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emax;


%Bootstrap

% Declare for storage
cL = NaN(totbins,R); 
cU = NaN(totbins,R);

disp('Bootstrapping')

parfor r = 1:R        
% Make the random weights 
WB = -spfun(@log,sprand(S)); relW = WB; relW(S0==0)=0;
relW = relW(:,1:totnonmiss);
WsumTot = sum(WB,2); WsumRel = sum(relW,2); 
WsumMiss = WsumTot - WsumRel;
   
    tau=.5;
    % Translate into a tau position.
    itau=int8(1+(tau-tau_l)/Dtau); %tau index        
        
    % Translate into a k position
    iK = int8(1+(K-K_l)/DK);

    % Calculate the upper and lower bounds for this replication
    cL(:,r) = f_boot_binbounds(relY, relW(:,:), tau*WsumTot(:,1) - (tau+K*(ones(totbins,1)-WsumMiss(:,1)./WsumTot(:,1))).*WsumMiss(:,1));
    cU(:,r) = f_boot_binbounds(relY, relW(:,:), tau*WsumTot(:,1) - (tau-K*(ones(totbins,1)-WsumMiss(:,1)./WsumTot(:,1))).*WsumMiss(:,1));

    
    %While we're here, let's get PT slopes
    % And finally use these to get the upper and lower bounds for pseudo
    LQ = cL(:,r); UQ = cU(:,r);

    % And find the beta lower and upper bounds
    [Bemin] = linprog( f, [], [], [], [], LQ, UQ, [], options);
    [Bemax] = linprog(-f, [], [], [], [], LQ, UQ, [], options);

    % Back out the pseudo true parameters implied by this and adjust approx
    Bpmin = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*Bemin;
    Bpmax = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*Bemax;
    slopeL(r) = Bpmin(1);
    slopeU(r) = Bpmax(1);        
end

%Confidence interval for approximate slope (ala Imbens and Manski)
disp('Confidence interval for pseudo-true slope')
CI=[quantile(slopeL,.05) quantile(slopeU,.95)]


%Get Imbens and Manski style critical value for CQF via Bonferroni
disp('Computing Critical Values')
alpherroni=1-(.95)^(1/totbins);

crit=norminv(1-alpherroni);
lb=NaN(totbins,1);
ub=NaN(totbins,1);
parfor b = 1:totbins    
    %get covariance of bounds in bin
    V=cov([cL(b,:)',cU(b,:)']);
    lb(b)=cLhat(b) - crit*sqrt(V(1,1)) - 5;
    ub(b)=cUhat(b) + crit*sqrt(V(2,2)) - 5;
end


bounds=[lb, ub];
boundshat=[cLhat-5, cUhat-5];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sixth: Compute / Output data for Graphs %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Bounds on schooling coefficient

%Parametric set

% Run the linprog on this thing - Set other parameters
f = [1 zeros(1, size(X,2)-1)];
A = [bins;-bins]; b = [ub;-lb]; Aeq = []; beq = [];
LQ = []; UQ = [];

% And find the beta lower and upper bounds
[bmin] = linprog( f, A, b, Aeq, beq, LQ, UQ,[],options);
[bmax] = linprog(-f, A, b, Aeq, beq, LQ, UQ,[],options);

[bmin bmax]

%Pseudo-true (BLA)

% Run the linprog on this thing - Set other parameters
f = [1 zeros(1, size(X,2)-1)]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
A = []; b = []; Aeq = []; beq = [];
LQ = lb; UQ = ub;

% And find the beta lower and upper bounds
[emin] = linprog( f, A, b, Aeq, beq, LQ, UQ,[],options);
[emax] = linprog(-f, A, b, Aeq, beq, LQ, UQ,[],options);

% Back out the pseudo true parameters implied by this and adjust approx
pmin = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emin;
pmax = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emax;

[pmin pmax]




%Set of Predictions
emin=NaN(totbins,1);
emax=NaN(totbins,1);
clear bmin bmax
options=optimset('LargeScale', 'off','display','off');

parfor i=1:totbins
f = bins(i,:)';
A = [bins;-bins]; b = [ub;-lb]; Aeq = []; beq = [];
LQ = []; UQ = [];

% And find the beta lower and upper bounds
bmin = linprog( f, A, b, Aeq, beq, LQ, UQ,[],options);
bmax = linprog(-f, A, b, Aeq, beq, LQ, UQ,[],options);
emin(i)=f'*bmin;
emax(i)=f'*bmax;
end



pemin=NaN(totbins,1);
pemax=NaN(totbins,1);
parfor i=1:totbins
% Run the linprog on this thing - Set other parameters
f = (bins(i,:)*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))')';
A = []; b = []; Aeq = []; beq = [];
LQ = lb; UQ = ub;

% And find the beta lower and upper bounds
pmin = linprog( f, A, b, Aeq, beq, LQ, UQ,[],options);
pmax = linprog(-f, A, b, Aeq, beq, LQ, UQ,[],options);
pemin(i)=f'*pmin;
pemax(i)=f'*pmax;
end




%get ready to graph

for i=1:totbins
    binref(i)=find(binind==i,1);
end



Z=[8 0 26 26^2 1];
idx1=binind(find(min(X==repmat(Z,length(X),1),[],2),1),:);
for i=9:20
    Z=[i 0 26 26^2 1];    
    idx1=[idx1;binind(find(min(X==repmat(Z,length(X),1),[],2),1),:)];
end


Z=[8 0 20 20^2 1];
idx2=binind(find(min(X==repmat(Z,length(X),1),[],2),1),:);
for i=9:20
    Z=[i 0 20 20^2 1];    
    idx2=[idx2;binind(find(min(X==repmat(Z,length(X),1),[],2),1),:)];
end



s=['~/Output/data4graph_' num2str(K,'% 10.2f') '.csv'];
es=[emin emax];
pes=[pemin pemax];
M1=[bins(idx1,1:3),bounds(idx1,:),boundshat(idx1,:),es(idx1,:),pes(idx1,:)];
M2=[bins(idx2,1:3),bounds(idx2,:),boundshat(idx2,:),es(idx2,:),pes(idx2,:)];
dlmwrite(s,[M1;M2]);




