%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Kline and Santos (2012)                  %
%     Code for Inference on Pseudo-True Parameter    %
%      Used for Figure 9 of paper             	     %
%  Fastest when run with parallel toolbox activated  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First: Declare parameters, get data %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

clear;

% Declare the parameters we will be using
R = 1000;          % Number of bootstrap replications.
alpha = 0.05;   % 1 - coverage for the confidence interval
Buff = 0.001;  % How much are we substracting from the sample kbar

% Reset the seed for the random number generator
rand('seed',0); rand('state',0);

% Declare quantile Range - and also k range + partition of grid
tau_l = .1; tau_u = .9;
Dtau = 0.05; DK = 0.01;
K_l = 0; K_u = NaN(int8(1+(tau_u-tau_l)/Dtau),1);

% Set linprog options
options = optimset('display','off'); options.MaxIter = 2000;

for yr=90:-10:80    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Second: Organize data, bins and bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

disp(['Loading and reshaping data for ', num2str(yr)])    

% Get data and back out the bins
% Note D is dummy for missing in code in paper D is dummy for nonmissing

eval(['[Y,X,D,n,dk] = getdata(' int2str(yr) ');']);

% Make Y >0 to enable sparse matrix calculations,  won't affect slope
Y = Y+5; 

% Figure out what are the demographic cells/bins
bins = unique(X, 'rows'); totbins = size(bins,1);
bincount = NaN(totbins,1); binind = zeros(n,1);
totobs = length(D); Mprob = NaN(totbins,1); 

% Figure out how many observations per bin, and who belongs where
parfor i = 1:totbins
    % How many observations per bin
    bincount(i) = sum(ismember(X,bins(i,:),'rows'));    
    binind = binind + (ismember(X,bins(i,:),'rows') == 1)*i;     
end

% And the missing probs per bin
parfor b = 1:totbins
    relD = D(binind == b);
    Mprob(b) = mean(relD);
end

% Create weights
weight = bincount/sum(bincount);
    
% Create a matrix relY that stacks bins in rows, obs in columns only
% contains observations of Ys that were not missing. Also bootstrap Ws
tot = max(bincount);
totnonmiss = int32(max((1-Mprob).*bincount));
relY = NaN(totbins,totnonmiss); 
S = NaN(totbins,tot);

% Preallocate bootstrap weights and put in array form
TempY = NaN; TempD = NaN;

% Reshape Y matrix into a row of nonmissing for each bin - pad with zeros
parfor b = 1:totbins
    % Back out the Ys that are not missing for this bin
    TempY = Y(binind == b); TempD = D(binind == b); TempY = sort(TempY(TempD == 0));

    % Store them
    S(b,:) = [ones(1,bincount(b)) zeros(1,tot - bincount(b))];
    relY(b,:) = [TempY' zeros(1,totnonmiss- length(TempY))];
end
relY = sparse(relY);
S = sparse(S);          % index of observations
S0 = spones(relY);      % index of nonmissing observations

% We'll use 1990 to figure out Ku(tau)
if yr == 90
    for i = 1:length(K_u)
        % Figure out who the relevant tau is and sample bounds
        reltau = tau_l + Dtau*(i-1);
        %sampbound = min([1-reltau reltau min((1-Mprob).*(1-reltau)./Mprob) min(reltau*(1-Mprob)./Mprob)]);
        %sampbound = min([1-reltau reltau min((1-reltau)./Mprob) min(reltau./Mprob)]);
        sampbound = min([min((1-reltau)./(ones(totbins,1)-Mprob)) min(reltau./(ones(totbins,1)-Mprob)) min((1-reltau)./Mprob) min(reltau./Mprob)]);
        K_u(i) = sampbound - Buff;
    end       
end

% And finally, we'll put an absolute upper bound of 0.3
K_u(K_u>= 0.3) = 0.3;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Third: Calculate the estimated bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Getting Sample Bounds for each quantile')
for tau=tau_l:Dtau:tau_u

% Translate into a tau position/index.
itau = int8(1+(tau-tau_l)/Dtau); 
        
% Figure out dimensions of what we are doing
if (itau==1)
    % Find out the total number of tau iterations
    maxitau = int8(1+(tau_u-tau_l)/Dtau);    
    maxiK = int16(1+(max(K_u)-K_l)/DK);    
    
    % Declare variables to save pseudo trues
    pmin = NaN(size(X,2),maxitau,maxiK);
    pmax = NaN(size(X,2),maxitau,maxiK);
end


for K=K_l:DK:K_u(itau)

% Translate into a k position/index
iK = int8(1+(K-K_l)/DK);

% Calculate the bounds at each cell
cL = f_binbounds(relY,tau - (tau+K*(ones(totbins,1)-Mprob)).*Mprob,bincount);
cU = f_binbounds(relY,tau - (tau-K*(ones(totbins,1)-Mprob)).*Mprob,bincount);

% If we asked for cs outside constraints, adjust
if any(isnan(cL))|any(isnan(cU))
    error('Sample bounds must be finite: lower k or restrict to narrower range of tau')
end

% Run the linprog on this thing - Set other parameters
f = [1 zeros(1, size(X,2)-1)]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
A = []; b = []; Aeq = []; beq = [];
LQ = cL; UQ = cU;

% And find the beta lower and upper bounds
[emin] = linprog( f, A, b, Aeq, beq, LQ, UQ,[],options);
[emax] = linprog(-f, A, b, Aeq, beq, LQ, UQ,[],options);

% Back out the pseudo true parameters implied by this and adjust approx
pmin(:,itau,iK) = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emin;
pmax(:,itau,iK) = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emax;

end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fourth: Bootstrap the procedure %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Declare for storage
BPBoundsL = NaN(R,maxitau,maxiK); 
BPBoundsU = NaN(R,maxitau,maxiK);

disp('Bootstrapping')

parfor r = 1:R
% Make the random weihts 
WB = -spfun(@log,sprand(S)); relW = WB; relW(S0==0)=0;
relW = relW(:,1:totnonmiss);
WsumTot = sum(WB,2); WsumRel = sum(relW,2); 
WsumMiss = WsumTot - WsumRel;
   
% Some temporary storage
TempL = NaN(maxitau,maxiK); TempU = NaN(maxitau,maxiK);

    for tau = tau_l:Dtau:tau_u
    % Translate into a tau position.
    itau=int8(1+(tau-tau_l)/Dtau); %tau index        
        
    for K = K_l:DK:K_u(itau)
    % Translate into a k position
    iK = int8(1+(K-K_l)/DK);
        
    % Calculate the upper and lower bounds for this replication
    cL = f_boot_binbounds(relY, relW(:,:), tau*WsumTot(:,1) - (tau+K*(ones(totbins,1)-WsumMiss(:,1)./WsumTot(:,1))).*WsumMiss(:,1));
    cU = f_boot_binbounds(relY, relW(:,:), tau*WsumTot(:,1) - (tau-K*(ones(totbins,1)-WsumMiss(:,1)./WsumTot(:,1))).*WsumMiss(:,1));

    if any(isnan(cL))|any(isnan(cU))
        warning('Infinite bootstrap bound detected: results may be unstable')
    end
    
    % And finally use these to get the upper and lower bounds for pseudo
    LQ = cL; UQ = cU;

    % And find the beta lower and upper bounds
    [Bemin] = linprog( f, [], [], [], [], LQ, UQ, [], options);
    [Bemax] = linprog(-f, [], [], [], [], LQ, UQ, [], options);

    % Back out the pseudo true parameters implied by this and adjust approx
    Bpmin = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*Bemin;
    Bpmax = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*Bemax;
    TempL(itau,iK) = Bpmin(1);
    TempU(itau,iK) = Bpmax(1);        
    end
    end
    
% And store the estimated upper lower bounds for this iteration
BPBoundsL(r,:,:) = TempL;
BPBoundsU(r,:,:) = TempU;            
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fifth: Construct the CI %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Use normal rule for quantile weighting function to stabilize variance.
% Other weighting schemes would may work as well.
scale = sqrt(normpdf(norminv([tau_l:Dtau:tau_u],0,1),0,1));

%Compute max deviation for uniform intervals upper and lower bounds
if yr==80
    % Create the confidence bound for upper envelope 1980
    TempBoot = -(shiftdim(BPBoundsU,1) - repmat(squeeze(pmax(1,:,:)),[1 1 R])).*repmat(scale',[1 size(BPBoundsL,3) R]);
    TempBoot = TempBoot(~isnan(TempBoot));
    R80 = quantile(TempBoot,1-alpha);
    C80 = squeeze(pmax(1,:,:)) + R80*repmat((1./scale)',[1 size(BPBoundsL,3)]);
    
    % Create the 2sided confidence bound for upper envelope 1980
    TempBoot = abs((shiftdim(BPBoundsU,1) - repmat(squeeze(pmax(1,:,:)),[1 1 R])).*repmat(scale',[1 size(BPBoundsL,3) R]));
    TempBoot = TempBoot(~isnan(TempBoot));
    R80_2S = quantile(TempBoot,1-alpha);
    C80_2SU = squeeze(pmax(1,:,:)) + R80_2S*repmat((1./scale)',[1 size(BPBoundsL,3)]);
    C80_2SL = squeeze(pmax(1,:,:)) - R80_2S*repmat((1./scale)',[1 size(BPBoundsL,3)]);
    
    % Save other things of interest
    pmin80 = squeeze(pmin(1,:,:));
    pmax80 = squeeze(pmax(1,:,:));
    
    clear TempBoot;
end    
    
if yr==90
    % Create the confidence bound for lower envelope 1990
    TempBoot = (shiftdim(BPBoundsL,1) - repmat(squeeze(pmin(1,:,:)),[1 1 R])).*repmat(scale',[1 size(BPBoundsL,3) R]);
    TempBoot = TempBoot(~isnan(TempBoot));
    R90 = quantile(TempBoot,1-alpha);
    C90 = squeeze(pmin(1,:,:)) - R90*repmat((1./scale)',[1 size(BPBoundsL,3)]);
    
    % Create the 2sided confidence bound for lower envelope 1990
    TempBoot = abs((shiftdim(BPBoundsL,1) - repmat(squeeze(pmin(1,:,:)),[1 1 R])).*repmat(scale',[1 size(BPBoundsL,3) R]));
    TempBoot = TempBoot(~isnan(TempBoot));
            
    R90_2S = quantile(TempBoot,1-alpha);
    C90_2SU = squeeze(pmin(1,:,:)) + R90_2S*repmat((1./scale)',[1 size(BPBoundsL,3)]);
    C90_2SL = squeeze(pmin(1,:,:)) - R90_2S*repmat((1./scale)',[1 size(BPBoundsL,3)]);
        
    % Save other things of interest
    pmin90 = squeeze(pmin(1,:,:));
    pmax90 = squeeze(pmax(1,:,:));
    
    clear TempBoot;    
end
end

% Get the index of the critical K for each tau - Lower envelope
CritK = NaN(maxitau,1);
parfor i = 1:maxitau
    rowdiff = C80(i,:) - C90(i,:);
    rowdiff = rowdiff(~isnan(rowdiff));
   
    if sum(rowdiff>=0) > 0
        CritK(i) = sum(rowdiff <0)*DK;
    end
end

% Get the index of the critical K for each tau - 2 sided
CritK_2S = NaN(maxitau,2);
for i = 1:maxitau
    % For calculating the lower bound
    rowdiffL = C80_2SU(i,:) - C90_2SL(i,:);
    rowdiffL = rowdiffL(~isnan(rowdiffL));
    
    % For calculating the upper bound
    rowdiffU = C80_2SL(i,:) - C90_2SU(i,:);
    rowdiffU = rowdiffU(~isnan(rowdiffU));
   
    % Find critical lower K
    if sum(rowdiffL>=0) > 0
        CritK_2S(i,1) = sum(rowdiffL <0)*DK;
    end
    
    % And critical upper K
    if sum(rowdiffU <= 0) > 0
        CritK_2S(i,2) = (sum(rowdiffU < 0)-1)*DK;
    end
end

% Get together in results
results = [(tau_l:Dtau:tau_u)' CritK]
results2S = [(tau_l:Dtau:tau_u)' CritK_2S];

save CritK8090 results results2S R80 R90 R80_2S R90_2S C80 C90 C80_2SU C80_2SL C90_2SU C90_2SL pmin80 pmax80 pmin90 pmax90 tau_l tau_u K_l K_u DK Dtau;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Sixth: Graph some of this stuff %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


pseudo_critical_getrefined_newks

clear;

load CritK8090; load refgrid;

% First get refined confidence intervals - Run this if we forgot to save
scale = sqrt(normpdf(norminv([tau_l:Dtau:tau_u],0,1),0,1));
R80 = mean((C80(:,1) - pmax80(:,1)).*scale');
R90 = mean((pmin90(:,1) - C90(:,1)).*scale');
R80_2S = mean((C80_2SU(:,1) - pmax80(:,1)).*scale');
R90_2S = mean((C90_2SU(:,1) - pmin90(:,1)).*scale');

% Now get the good confidence intervals
thinscale = sqrt(normpdf(norminv(thintau,0,1),0,1)); 
thinC80 = thinpmax80 + R80*repmat(1./thinscale, [1 size(thinpmax80,2)]);
thinC90 = thinpmin90 - R90*repmat(1./thinscale, [1 size(thinpmin90,2)]);
thinC80_2SL = thinpmax80 - R80_2S*repmat(1./thinscale, [1 size(thinpmax80,2)]);
thinC80_2SU = thinpmax80 + R80_2S*repmat(1./thinscale, [1 size(thinpmax80,2)]);
thinC90_2SL = thinpmin90 - R90_2S*repmat(1./thinscale, [1 size(thinpmin90,2)]);
thinC90_2SU = thinpmin90 + R90_2S*repmat(1./thinscale, [1 size(thinpmin90,2)]);

% And use this to fine an improved K
thinCritK = NaN(length(thintau),1); thinCritK_2S = NaN(length(thintau),2);
for i = 1:length(thintau)
    rowdiff = thinpmax80(i,:) - thinpmin90(i,:); rowdiff = rowdiff(~isnan(rowdiff));
    rowdiffL = thinC80_2SU(i,:) - thinC90_2SL(i,:); rowdiffL = rowdiffL(~isnan(rowdiffL));
    rowdiffU = thinC80_2SL(i,:) - thinC90_2SU(i,:); rowdiffU = rowdiffU(~isnan(rowdiffU));
    
    % Find the critical K (point estimate)
    if sum(rowdiff>=0) > 0
        thinCritK(i) = sum(rowdiff <0)*thinK(2);
    end
    % Find critical lower K (2 sided)
    if sum(rowdiffL>=0) > 0
        thinCritK_2S(i,1) = sum(rowdiffL <0)*thinK(2);
    end
    % And critical upper K  (2 sided)
    if sum(rowdiffU <= 0) > 0
        thinCritK_2S(i,2) = (sum(rowdiffU < 0)-1)*thinK(2);
    end
end

% See if we can graph some of this stuff - First refine the gridpoints

critcross = thintau*NaN; critcrossU=critcross; critcrossL=critcross;


% Get the level corresponding to critical k and tau
for i = 1:size(thinC80,1)
    critcross(i) = (thinpmin90(i,round(thinCritK(i)/thinK(2)) + 1) + thinpmax80(i,round(thinCritK(i)/thinK(2)) + 1))/2;
    critcrossL(i) = (thinC90_2SL(i,round(thinCritK(i)/thinK(2)) + 1) + thinC80_2SU(i,round(thinCritK(i)/thinK(2)) + 1))/2;
    critcrossU(i) = (thinC90_2SU(i,round(thinCritK(i)/thinK(2)) + 1) + thinC80_2SL(i,round(thinCritK(i)/thinK(2)) + 1))/2;
end


% And plot everything
figure(1)
clf
mesh(thintau, thinK, thinpmax80');
xlabel tau; ylabel k; zlabel dY/dE;
title('1980 Sample Upper Bound')
print -depsc2 -r150 ~/Output/mesh80.eps
system('ps2pdf -dEPSCrop ~/Output/mesh80.eps ~/Output/mesh80.pdf')

clf
mesh(thintau, thinK, thinpmin90');
xlabel tau; ylabel k; zlabel dY/dE;
title('1990 Sample Lower Bound')
print -depsc2 -r150 ~/Output/mesh90.eps
system('ps2pdf -dEPSCrop ~/Output/mesh90.eps ~/Output/mesh90.pdf')

clf
mesh(thintau, thinK, thinpmax80'); hold;
mesh(thintau, thinK, thinpmin90');

scatter3(thintau, thinCritK, critcross, 'o','black','filled')
xlabel tau; ylabel k; zlabel dY/dE;
title('Intersection of Sample Bounds')
annotation('textarrow',[0.7698 0.5851],[0.3593 0.5492],'string','Breakdown Function')
view([-1,-1,1])
print -depsc2 -r150 ~/Output/mesh8090.eps
system('ps2pdf -dEPSCrop ~/Output/mesh8090.eps ~/Output/mesh8090.pdf')


%Giant picture
%clf
%mesh(thintau, thinK, thinpmax80'); hold;
%mesh(thintau, thinK, thinC80_2SU');
%mesh(thintau, thinK, thinC80_2SL');
%xlabel tau; ylabel k; zlabel dY/dE;
%title('1980 Sample Upper Bounds')
%print -depsc2 -r150 ~/Output/meshs80.eps
%system('ps2pdf ~/Output/meshs80.eps ~/Output/meshs80.pdf')

%mesh(thintau, thinK, thinpmin90');
%mesh(thintau, thinK, thinC90_2SU');
%mesh(thintau, thinK, thinC90_2SL');

%scatter3(thintau, thinCritK, critcross, 'o','black','filled')
%scatter3(thintau, thinCritK_2S(:,1), critcrossL, 'o','blue','filled')
%scatter3(thintau, thinCritK_2S(:,2), critcrossU, 'o','red','filled')
%xlabel tau; ylabel k; zlabel dY/dE;
%title('Intersection of Bounds')
%annotation('textarrow',[0.7698 0.5851],[0.3593 0.5492],'string','Breakdown Function')


M(1)=getframe(gcf);
for i=1:100
    if i<=50
        view([-1+i/50,-1,1])
    else
        view([0,-1+(i-50)/50,1])
    end
    M(i+1)=getframe(gcf);
end

clf
plot(thintau, thinCritK_2S(:,1),'--g',thintau,thinCritK,'-k',thintau,.12*ones(length(thintau),1),'--c',thintau, thinCritK_2S(:,2), '--g','linewidth',2);
%colormap('gray')
xlabel tau; ylabel k;
legend('Lower Bound', 'Point Estimate', 'Upper Bound','Location','South')
title('Breakdown Curve (1980 vs 1990)')
M(i+1)=getframe(gcf);
for k=2:20
M(i+k)=M(i+1);
end


print -depsc2 -r150 ~/Output/breakdown2.eps %updated to add horizontal line
system('ps2pdf -dEPSCrop ~/Output/breakdown2.eps ~/Output/breakdown2.pdf')

movie2avi(M,'~/Output/movie8090.avi')


