%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Kline and Santos (2012)                  %
%     Code for Inference on Pseudo-True Parameter    %
%      Used for Figure 9 of paper             	     %
%  Fastest when run with parallel toolbox activated  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First: Declare parameters, get data %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

clear;

% Declare the parameters we will be using
Buff = 0.001;  % How much are we substracting from the sample kbar

% Reset the seed for the random number generator
rand('seed',0); rand('state',0);

% Declare quantile Range - and also k range + partition of grid
tau_l = .1; tau_u = .9;
Dtau = 0.01; DK = 0.001;
K_l = 0; K_u = NaN(int16(1+(tau_u-tau_l)/Dtau),1);

% And make a vector of some of these
thintau = (tau_l:Dtau:tau_u)'; maxitau = length(thintau);

% Set linprog options
options = optimset('display','off'); options.MaxIter = 2000;

for yr=90:-10:80    

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Second: Organize data, bins and bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

disp(['Loading and reshaping data for ', num2str(yr)])    

% Get data and back out the bins
% Note D is dummy for missing in code in paper D is dummy for nonmissing

eval(['[Y,X,D,n,dk] = getdata(' int2str(yr) ');']);

% Make Y >0 to enable sparse matrix calculations,  won't affect slope
Y = Y+5; 

% Figure out what are the demographic cells/bins
bins = unique(X, 'rows'); totbins = size(bins,1);
bincount = NaN(totbins,1); binind = zeros(n,1);
totobs = length(D); Mprob = NaN(totbins,1); 

% Figure out how many observations per bin, and who belongs where
parfor i = 1:totbins
    % How many observations per bin
    bincount(i) = sum(ismember(X,bins(i,:),'rows'));    
    binind = binind + (ismember(X,bins(i,:),'rows') == 1)*i;     
end

% And the missing probs per bin
parfor b = 1:totbins
    relD = D(binind == b);
    Mprob(b) = mean(relD);
end

% Create weights
weight = bincount/sum(bincount);
    
% Create a matrix relY that stacks bins in rows, obs in columns only
% contains observations of Ys that were not missing. Also bootstrap Ws
tot = max(bincount);
totnonmiss = int32(max((1-Mprob).*bincount));
relY = NaN(totbins,totnonmiss); 
S = NaN(totbins,tot);

% Preallocate bootstrap weights and put in array form
TempY = NaN; TempD = NaN;

% Reshape Y matrix into a row of nonmissing for each bin - pad with zeros
parfor b = 1:totbins
    % Back out the Ys that are not missing for this bin
    TempY = Y(binind == b); TempD = D(binind == b); TempY = sort(TempY(TempD == 0));

    % Store them
    S(b,:) = [ones(1,bincount(b)) zeros(1,tot - bincount(b))];
    relY(b,:) = [TempY' zeros(1,totnonmiss- length(TempY))];
end
relY = sparse(relY);
S = sparse(S);          % index of observations
S0 = spones(relY);      % index of nonmissing observations

% We'll use 1990 to figure out Ku(tau)
if yr == 90
    for i = 1:length(K_u)
        % Figure out who the relevant tau is and sample bounds
        reltau = tau_l + Dtau*(i-1);
        %sampbound = min([1-reltau reltau min((1-Mprob).*(1-reltau)./Mprob) min(reltau*(1-Mprob)./Mprob)]);
        %sampbound = min([1-reltau reltau min((1-reltau)./Mprob) min(reltau./Mprob)]);
        sampbound = min([min((1-reltau)./(ones(totbins,1)-Mprob)) min(reltau./(ones(totbins,1)-Mprob)) min((1-reltau)./Mprob) min(reltau./Mprob)]);
    K_u(i) = sampbound - Buff;
    end       
end

% And finally, we'll put an absolute upper bound of 0.3
K_u(K_u>= 0.3) = 0.3;
thinK = (K_l:DK:max(K_u))';
maxiK = length(thinK);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Third: Calculate the estimated bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Getting Sample Bounds for each quantile')

pmin = NaN(maxitau,maxiK);
pmax = NaN(maxitau,maxiK);

for itau= 1:maxitau
        
% Set up easy index
reltau = thintau(itau);

parfor iK = 1:round(K_u(itau)/DK+1)

% Set up easy notation
relK = thinK(iK);
    
% Calculate the bounds at each cell

cL = f_binbounds(relY,reltau - (reltau+relK*(ones(totbins,1)-Mprob)).*Mprob,bincount);
cU = f_binbounds(relY,reltau - (reltau-relK*(ones(totbins,1)-Mprob)).*Mprob,bincount);

% If we asked for cs outside constraints, adjust
if any(isnan(cL))|any(isnan(cU))
    error('Sample bounds must be finite: lower k or restrict to narrower range of tau')
end

% Run the linprog on this thing - Set other parameters
f = [1 zeros(1, size(X,2)-1)]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
A = []; b = []; Aeq = []; beq = [];
LQ = cL; UQ = cU;

% And find the beta lower and upper bounds
[emin] = linprog( f, A, b, Aeq, beq, LQ, UQ,[],options);
[emax] = linprog(-f, A, b, Aeq, beq, LQ, UQ,[],options);

% Back out the pseudo true parameters implied by this and adjust approx
temppmin = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emin;
temppmax = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emax;

pmin(itau,iK) = temppmin(1); pmax(itau,iK) = temppmax(1);
end
end

% Save the pmins and pmax
if yr == 80
    thinpmin80 = pmin; thinpmax80 = pmax;
else
    thinpmin90 = pmin; thinpmax90 = pmax;
end
end

save refgrid thinpmin80 thinpmin90 thinpmax80 thinpmax90 thinK thintau;