%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               Kline and Santos (2011)              %
%   Code for Inference on Fitted Pseudo-True Values  %
%             Used for Figure 10 of paper            %
%  Fastest when run with parallel toolbox activated  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First: Declare parameters, get data %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

clear;

% Declare the parameters we will be using
R = 1000;       % Number of bootstrap replications.
K = 0.02;       % The constraint in the usual sense.
big = 1;        % 1 - lemieux specification, 0 - ACF
alpha = 0.05;   % 1 - coverage for the confidence interval

% Declare quantiles of interest and values of education
qs = [.1 .5 .9]; educ=[8:20]';

% Reset the seed for the random number generator
rand('seed',0); rand('state',0);

% Set linprog options
options = optimset('display','off'); options.MaxIter = 2000;

for yr=80:10:100

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Second: Organize data, bins and bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

disp(['Loading and reshaping data for ', num2str(yr)])    

% Get data and back out the bins
% Note D is dummy for missing in code in paper D is dummy for nonmissing

eval(['[Y,X,D,n,dk]=getdata(' int2str(yr) ');']);

% Make Y >0 to enable sparse matrix calculations,  won't affect slope
Y=Y+5;

if big==1
    % Recenter the regressors and square
    X(:,3) = X(:,3)-mean(X(:,3)); X(:,4) = X(:,3).^2;
   
    % Quadratic in edu, race dummy, quartic in potential xp., intercept
    X=[X(:,1), X(:,1).^2, X(:,[2 3 4]), X(:,3).^3, X(:,3).^4, X(:,5)]; 
end

% Figure out what the demographic cells are
bins = unique(X, 'rows'); totbins = size(bins,1);
bincount = NaN(totbins,1); binind = zeros(n,1);
totobs = length(D); Mprob = NaN(totbins,1); 

% Figure out how many observations per bin, and who belongs where
parfor i = 1:totbins
    % How many observations per bin
    bincount(i) = sum(ismember(X,bins(i,:),'rows'));    
    binind = binind + (ismember(X,bins(i,:),'rows') == 1)*i;     
end

% And the missing probs per bin
parfor b = 1:totbins
    relD = D(binind == b);
    Mprob(b) = mean(relD);
end

% Create weights (measure S)
weight = bincount/sum(bincount);

% Create a matrix relY that stacks bins in rows, obs in columns only
% contains observations of Ys that were not missing. Also bootstrap Ws
tot = max(bincount);
totnonmiss = int32(max((1-Mprob).*bincount));
relY = NaN(totbins,totnonmiss); 
S = NaN(totbins,tot);

% Preallocate bootstrap weights and put in array form
TempY = NaN; TempD = NaN;

% Reshape Y matrix into a row of nonmissing for each bin - pad with zeros
parfor b = 1:totbins
    % Back out the Ys that are not missing for this bin
    TempY = Y(binind == b); TempD = D(binind == b); TempY = sort(TempY(TempD == 0));

    % Store them
    S(b,:) = [ones(1,bincount(b)) zeros(1,tot - bincount(b))];
    relY(b,:) = [TempY' zeros(1,totnonmiss- length(TempY))];
end
relY = sparse(relY);
S = sparse(S);          %index of observations
S0 = spones(relY);      %index of nonmissing observations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Third: Calculate the estimated bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Getting Sample Bounds for each quantile')

for tau=qs

% Translate into a tau position/index
itau = find(qs==tau);
maxitau = length(qs);

if itau==1       
    % Declare variables to save pseudo trues
    pmin = NaN(size(X,2),maxitau,length(educ));
    pmax = NaN(size(X,2),maxitau,length(educ));
end

% Calculate the bounds at each cell
cL = f_binbounds(relY,tau - (tau+K*(ones(totbins,1)-Mprob)).*Mprob,bincount);
cU = f_binbounds(relY,tau - (tau-K*(ones(totbins,1)-Mprob)).*Mprob,bincount);

% If we asked for cs outside constraints, adjust
if any(isnan(cL))|any(isnan(cU))
    error('Sample bounds must be finite: lower k or restrict to narrower range of tau')
end


% Run the linprog on this thing - Set other parameters
f = [1 zeros(1, size(X,2)-1)]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
A = []; b = []; Aeq = []; beq = [];
LQ = cL; UQ = cU;

% Set other covariates besides first at mean
if big==0
    xbar=mean(X(:,[2:5]));
else
    xbar=mean(X(:,[3:8]));
end

for j=educ'
    if big==0
        f = [j xbar]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
    else
        f = [j j^2 xbar]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
    end

    % And find the beta lower and upper bounds
    [emin] = linprog( f, A, b, Aeq, beq, LQ, UQ,[],options);
    [emax] = linprog(-f, A, b, Aeq, beq, LQ, UQ,[],options);

    % Back out the pseudo true parameters implied by this and adjust approx
    pmin(:,itau,1+j-educ(1)) = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emin;
    pmax(:,itau,1+j-educ(1)) = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emax;
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fourth: Bootstrap the procedure %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Declare for storage
BPBoundsL = NaN(R,length(educ),maxitau); 
BPBoundsU = NaN(R,length(educ),maxitau);

disp('Bootstrapping')

for r = 1:R
% Make the random weihts 
WB = -spfun(@log,sprand(S)); relW = WB; relW(S0==0)=0;
relW = relW(:,1:totnonmiss);
WsumTot = sum(WB,2); WsumRel = sum(relW,2); 
WsumMiss = WsumTot - WsumRel;
   
% Some temporary storage
TempL = NaN(length(educ),maxitau); TempU = NaN(length(educ),maxitau);

for tau=qs
    % Translate into a tau position/index
    itau=find(qs==tau);
            
    % Calculate the upper and lower bounds for this replication
    cL = f_boot_binbounds(relY, relW(:,:), tau*WsumTot(:,1) - (tau+K*(ones(totbins,1)-WsumMiss(:,1)./WsumTot)).*WsumMiss(:,1));
    cU = f_boot_binbounds(relY, relW(:,:), tau*WsumTot(:,1) - (tau-K*(ones(totbins,1)-WsumMiss(:,1)./WsumTot)).*WsumMiss(:,1));

    if any(isnan(cL))|any(isnan(cU))
        warning('Infinite bootstrap bound detected: results may be unstable')
    end
    
    % And finally use these to get the upper and lower bounds for pseudo
    LQ = cL; UQ = cU;
    BPBoundsLvec=NaN(length(educ),1);
    BPBoundsUvec=NaN(length(educ),1);
    
    for j=educ'
        edind=find(educ==j);

        if big==0
            f = [j xbar]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
        else
            f = [j j^2 xbar]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
        end
        
        % And find the beta lower and upper bounds
        [Bemin] = linprog( f, [], [], [], [], LQ, UQ,[],options);
        [Bemax] = linprog(-f, [], [], [], [], LQ, UQ,[],options);

        % Back out the pseudo true parameters implied by this and adjust approx
        Bpmin = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*Bemin;
        Bpmax = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*Bemax;
        if big==0
            BPBoundsLvec(edind) = [j xbar]*Bpmin;
            BPBoundsUvec(edind) = [j xbar]*Bpmax;
        else
            BPBoundsLvec(edind) = [j j^2 xbar]*Bpmin;
            BPBoundsUvec(edind) = [j j^2 xbar]*Bpmax;
        end
    end
    TempL(:,itau)=BPBoundsLvec';
    TempU(:,itau)=BPBoundsUvec';
end
    
% And store the upper and lower bound results
BPBoundsL(r,:,:) = TempL;
BPBoundsU(r,:,:) = TempU;            
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fifth: Construct the CI %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

for t=1:maxitau
    for j=educ'    
        if big==0
            Cl=quantile(max([(BPBoundsL(:,1+j-educ(1),t) - ones(R,1)*[j xbar]*pmin(:,t,1+j-educ(1)))],[],2),1-alpha);
            Cu=quantile(max([-(BPBoundsU(:,1+j-educ(1),t) - ones(R,1)*[j xbar]*pmax(:,t,1+j-educ(1)))],[],2),1-alpha);
        else
            Cl=quantile(max([(BPBoundsL(:,1+j-educ(1),t) - ones(R,1)*[j j^2 xbar]*pmin(:,t,1+j-educ(1)))],[],2),1-alpha);
            Cu=quantile(max([-(BPBoundsU(:,1+j-educ(1),t) - ones(R,1)*[j j^2 xbar]*pmax(:,t,1+j-educ(1)))],[],2),1-alpha);
        end
    if big==0
        CI_l(t,1+j-educ(1)) = [j xbar]*pmin(:,t,1+j-educ(1))-Cl; 
        CI_u(t,1+j-educ(1)) = [j xbar]*pmax(:,t,1+j-educ(1))+Cu; 
    else
        CI_l(t,1+j-educ(1)) = [j j^2 xbar]*pmin(:,t,1+j-educ(1))-Cl; 
        CI_u(t,1+j-educ(1)) = [j j^2 xbar]*pmax(:,t,1+j-educ(1))+Cu; 
    end
    end
end

% Recenter intervals (see line 48)
CI_l = CI_l-5; CI_u = CI_u-5;

% Stack results
if yr==80
    results=[yr*ones(maxitau*length(educ),1), kron(ones(maxitau,1),educ), kron(qs',ones(length(educ),1)), reshape(CI_l',[],1), reshape(CI_u',[],1)]
else
    results=[results; yr*ones(maxitau*length(educ),1), kron(ones(maxitau,1),educ), kron(qs',ones(length(educ),1)), reshape(CI_l',[],1), reshape(CI_u',[],1)];
end
end

% Break up these results
results80_q1 = results(results(:,1)==80&results(:,3)==qs(1),:)
results80_q2 = results(results(:,1)==80&results(:,3)==qs(2),:)
results80_q3 = results(results(:,1)==80&results(:,3)==qs(3),:)

results90_q1 = results(results(:,1)==90&results(:,3)==qs(1),:)
results90_q2 = results(results(:,1)==90&results(:,3)==qs(2),:)
results90_q3 = results(results(:,1)==90&results(:,3)==qs(3),:)

results100_q1 = results(results(:,1)==100&results(:,3)==qs(1),:)
results100_q2 = results(results(:,1)==100&results(:,3)==qs(2),:)
results100_q3 = results(results(:,1)==100&results(:,3)==qs(3),:)

% Rescale k and save
k=K*100;

eval(['save results_' int2str(k) '_' int2str(big) '_PT_levels_fitted results;']);
eval(['save -ascii -tabs results_' int2str(k) '_' int2str(big) '_PT_levels_fitted.txt results;']); 

% Graph the output - figure 1
figure(1)
plot(results80_q1(:,2),results80_q1(:,4),results80_q1(:,2),results80_q1(:,5),results80_q2(:,2),results80_q2(:,4),results80_q2(:,2),results80_q2(:,5),results80_q3(:,2),results80_q3(:,4),results80_q3(:,2),results80_q3(:,5))
title('Bounds on Pseudo-True Returns to Schooling 1980')
ylabel('lnE')
xlabel('Years of Schooling')
legend('tau=.1','tau=.1','tau=.5','tau=.5','tau=.9', 'tau=.9', 'Location','BestOutside')

% Figure 2
figure(2)
plot(results90_q1(:,2),results90_q1(:,4),results90_q1(:,2),results90_q1(:,5),results90_q2(:,2),results90_q2(:,4),results90_q2(:,2),results90_q2(:,5),results90_q3(:,2),results90_q3(:,4),results90_q3(:,2),results90_q3(:,5))
title('Bounds on Pseudo-True Returns to Schooling 1990')
ylabel('lnE')
xlabel('Years of Schooling')
legend('tau=.1','tau=.1','tau=.5','tau=.5','tau=.9', 'tau=.9', 'Location','BestOutside')

% Figure 3
figure(3)
plot(results100_q1(:,2),results100_q1(:,4),results100_q1(:,2),results100_q1(:,5),results100_q2(:,2),results100_q2(:,4),results100_q2(:,2),results100_q2(:,5),results100_q3(:,2),results100_q3(:,4),results100_q3(:,2),results100_q3(:,5))
title('Bounds on Pseudo-True Returns to Schooling 2000')
ylabel('lnE')
xlabel('Years of Schooling')
legend('tau=.1','tau=.1','tau=.5','tau=.5','tau=.9', 'tau=.9', 'Location','BestOutside')

% Save the file and export
file=['~/Output/BF_PT_level_' int2str(k) '_' int2str(big)];
hgexport(1,file)
