%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%           Kline and Santos (2012)                  %
%     Code for Inference on Pseudo-True Parameter    %
%      Used for Figures 6, 7, and 8 of paper         %
%  Fastest when run with parallel toolbox activated  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% First: Declare parameters, get data %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

clear;

% Declare the parameters we will be using
R = 1000;       % Number of bootstrap replications.
K = 0.02;      % The constraint in the usual sense.
alpha = 0.05;   % 1 - coverage for the confidence interval

% Reset the seed for the random number generator
rand('seed',0); rand('state',0);

% Declare quantile Range
tau_l = .1; tau_u = .9;

% Set linprog options
options = optimset('display','off'); options.MaxIter = 2000;

for yr=80:10:100

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Second: Organize data, bins and bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%    

disp(['Loading and reshaping data for ', num2str(yr)])    

% Get data and back out the bins
% Note D is dummy for missing in code in paper D is dummy for nonmissing

eval(['[Y,X,D,n,dk] = getdata(' int2str(yr) ');']);

% Make Y >0 to enable sparse matrix calculations,  won't affect slope
Y = Y+5; 

% Figure out what are the demographic cells/bins
bins = unique(X, 'rows'); totbins = size(bins,1);
bincount = NaN(totbins,1); binind = zeros(n,1);
totobs = length(D); Mprob = NaN(totbins,1); 

% Figure out how many observations per bin, and who belongs where
parfor i = 1:totbins
    % How many observations per bin
    bincount(i) = sum(ismember(X,bins(i,:),'rows'));    
    binind = binind + (ismember(X,bins(i,:),'rows') == 1)*i;     
end

% And the missing probs per bin
parfor b = 1:totbins
    relD = D(binind == b);
    Mprob(b) = mean(relD);
end

% Create weights
weight = bincount/sum(bincount);
    
% Create a matrix relY that stacks bins in rows, obs in columns only
% contains observations of Ys that were not missing. Also bootstrap Ws
tot = max(bincount);
totnonmiss = int32(max((1-Mprob).*bincount));
relY = NaN(totbins,totnonmiss); 
S = NaN(totbins,tot);

% Preallocate bootstrap weights and put in array form
TempY = NaN; TempD = NaN;

% Reshape Y matrix into a row of nonmissing for each bin - pad with zeros
parfor b = 1:totbins
    % Back out the Ys that are not missing for this bin
    TempY = Y(binind == b); TempD = D(binind == b); TempY = sort(TempY(TempD == 0));

    % Store them
    S(b,:) = [ones(1,bincount(b)) zeros(1,tot - bincount(b))];
    relY(b,:) = [TempY' zeros(1,totnonmiss- length(TempY))];
end
relY = sparse(relY);
S = sparse(S);          % index of observations
S0 = spones(relY);      % index of nonmissing observations

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Third: Calculate the estimated bounds %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

disp('Getting Sample Bounds for each quantile')
for tau=tau_l:.05:tau_u

% Translate into a tau position/index.
itau = int8(1+(tau-tau_l)/.05); 

if itau==1    
    % Find out the total number of tau iterations
    maxitau = int8(1+(tau_u-tau_l)/.05);
    
    % Declare variables to save pseudo trues
    pmin = NaN(size(X,2),maxitau);
    pmax = NaN(size(X,2),maxitau);
end

% Calculate the bounds at each cell
cL = f_binbounds(relY,tau - ((tau+K*(ones(totbins,1)-Mprob)).*Mprob),bincount);
cU = f_binbounds(relY,tau - ((tau-K*(ones(totbins,1)-Mprob)).*Mprob),bincount);

% If we asked for cs outside constraints, adjust
if any(isnan(cL))|any(isnan(cU))
    error('Sample bounds must be finite: lower k or restrict to narrower range of tau')
end

% Run the linprog on this thing - Set other parameters
f = [1 zeros(1, size(X,2)-1)]*inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))';
A = []; b = []; Aeq = []; beq = [];
LQ = cL; UQ = cU;

% And find the beta lower and upper bounds
[emin] = linprog( f, A, b, Aeq, beq, LQ, UQ,[],options);
[emax] = linprog(-f, A, b, Aeq, beq, LQ, UQ,[],options);

% Back out the pseudo true parameters implied by this and adjust approx
pmin(:,itau) = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emin;
pmax(:,itau) = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*emax;

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fourth: Bootstrap the procedure %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Declare for storage
BPBoundsL = NaN(R,maxitau); 
BPBoundsU = NaN(R,maxitau);

disp('Bootstrapping')

parfor r = 1:R
% Make the random weihts 
WB = -spfun(@log,sprand(S)); relW = WB; relW(S0==0)=0;
relW = relW(:,1:totnonmiss);
WsumTot = sum(WB,2); WsumRel = sum(relW,2); 
WsumMiss = WsumTot - WsumRel;
   
% Some temporary storage
TempL = NaN(maxitau,1); TempU = NaN(maxitau,1);

    for tau = tau_l:.05:tau_u
    % Translate into a tau position.
    itau=int8(1+(tau-tau_l)/.05); %tau index        
        
    % Calculate the upper and lower bounds for this replication
    cL = f_boot_binbounds(relY, relW(:,:), tau*WsumTot(:,1) - (tau+K*(ones(totbins,1)-WsumMiss(:,1)./WsumTot(:,1))).*WsumMiss(:,1));
    cU = f_boot_binbounds(relY, relW(:,:), tau*WsumTot(:,1) - (tau-K*(ones(totbins,1)-WsumMiss(:,1)./WsumTot(:,1))).*WsumMiss(:,1));

    if any(isnan(cL))|any(isnan(cU))
        warning('Infinite bootstrap bound detected: results may be unstable')
    end
    
    % And finally use these to get the upper and lower bounds for pseudo
    LQ = cL; UQ = cU;

    % And find the beta lower and upper bounds
    [Bemin] = linprog( f, [], [], [], [], LQ, UQ, [], options);
    [Bemax] = linprog(-f, [], [], [], [], LQ, UQ, [], options);

    % Back out the pseudo true parameters implied by this and adjust approx
    Bpmin = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*Bemin;
    Bpmax = inv((bins.*repmat(weight,1,size(bins,2)))'*bins)*(bins.*repmat(weight,1,size(bins,2)))'*Bemax;
    TempL(itau) = Bpmin(1);
    TempU(itau) = Bpmax(1);        
    end
    
% And store the estimated upper lower bounds for this iteration
BPBoundsL(r,:) = TempL;
BPBoundsU(r,:) = TempU;            
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Fifth: Construct the CI %
%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Use normal rule for quantile weighting function to stabilize variance.
% Other weighting schemes would may work as well.
scale = sqrt(normpdf(norminv([tau_l:.05:tau_u],0,1),0,1));

%Compute max deviation for uniform intervals
C = quantile(max([(BPBoundsL - ones(R,1)*pmin(1,:)).*(ones(R,1)*scale),-(BPBoundsU - ones(R,1)*pmax(1,:)).*(ones(R,1)*scale)],[],2),1-alpha);
CI = NaN(2,maxitau);
CI(1,:) = pmin(1,:)-C./scale; 
CI(2,:) = pmax(1,:)+C./scale; 

%Stack Results
if yr==80
    results = [yr*ones(maxitau,1), (tau_l:.05:tau_u)', CI']
else
    results = [results; yr*ones(maxitau,1), (tau_l:.05:tau_u)', CI'];
end
end

results80 = results(results(:,1)==80,:)
results90 = results(results(:,1)==90,:)
results100= results(results(:,1)==100,:)

k=K*100;

% Save all these results
eval(['save results_' int2str(k) '_PT_levels results;']);
eval(['save -ascii -tabs results_' int2str(k) '_PT_levels.txt results;']); 

% And create the plots of the bounds
plot(results80(:,2),results80(:,3),results80(:,2),results80(:,4),results90(:,2),results90(:,3),results90(:,2),results90(:,4),results100(:,2),results100(:,3),results100(:,2),results100(:,4))
title('Bounds on Pseudo-True Returns to Schooling 1980-2000')
ylabel('dlnE / dS')
xlabel('Tau')
legend('80 LB','80 UB','90 LB','90 UB','2000 LB', '2000 UB', 'Location','BestOutside')

% Export this file to a different directory
file=['~/Output/BF_PT_level_' int2str(k)];
hgexport(1,file)
